from sklearn_compat._sklearn_compat import (
    is_df_or_series,  # noqa: F401
    is_pandas_df_or_series,  # noqa: F401
    is_pandas_df,  # noqa: F401
    is_polars_df_or_series,  # noqa: F401
    is_polars_df,  # noqa: F401
    is_pyarrow_data,  # noqa: F401
)